﻿class GnCleaner {
    GnCleaner() {}

    # 属性
    [string]$Property1

    [string[]] getDockerIds() {

        $ids = Invoke-Expression "docker ps -q"
        return $ids

    }

    # 返回本地服务的 url
    [string] getLocalWebAppUrl() {
        
        $ports = @(80, 81 )

        foreach ($port in $ports) {
            $status = Test-NetConnection '127.0.0.1' -Port $port
            if ($status.TcpTestSucceeded) {
                return "http://127.0.0.1:$port"
            }
        }

        throw [System.InvalidOperationException]::new("无法定位本地 web 地址 ")
      
    }

    [object] getRunningWxDockerInfos() {

        $urlhost = $this.getLocalWebAppUrl()

        $url = "$urlhost/gnapi/Test/GetOnlineWxs"

        $this.showlog("本地地址：$url")

        $response = Invoke-WebRequest -Uri $url
        $json = $response.Content | ConvertFrom-Json

        return $json
    }

    # 开始删除文件 
    [void] deleteFolders([string]$root, [string[]]$hostNames){

        $this.showlog("处理目录 $root")    

        Get-ChildItem -Path $root -Directory | ForEach-Object {
            
            $name = $_.Name

            if ($hostNames -contains $name) {
                # Write-Output "保留容器：$guid"   
                # $this.showlog("保留 $name")    
            }
            else {
                $fullPath = $_.FullName
                $this.showlog("删除 $fullPath")     
                Remove-Item -Path $fullPath -Recurse -Force
            }
          
        }
    }

    [void] clearLogs() {

        $this.showlog("开始清理日志...")   

        $json = $this.getRunningWxDockerInfos()

    
        # $hostNames = $json.data.data | ForEach-Object Guid  

        $hostNames = $json.data.data | ForEach-Object HostName
    
        $hostNames | ForEach-Object { $this.showlog("工作中 hostName: $_") }

        $root = 'D:\gnlab\gnproxy-logs'

        $this.deleteFolders($root, $hostNames)
    }

    [void] clearWxWorkFiles(){
        
        $this.showlog("开始清理企业微信文件...")   
        
        $json = $this.getRunningWxDockerInfos()

        $hostNames = $json.data.data | ForEach-Object Guid  
    
        $hostNames | ForEach-Object { $this.showlog("工作中 guid: $_") }

        $root = 'D:\gnlab\gnproxy'

        $this.deleteFolders($root, $hostNames)

    }

    [void] ClearDockers() {

        $this.showlog("开始清理容器...")

        # Invoke-Expression "docker system prune -a" # 未使用的镜像也会被清理 
        Invoke-Expression "docker system prune -f"
    }

    # 方法
    [void] clear() {

        $this.showlog("开始清理...")

        $this.ClearDockers()

        $this.clearLogs()

        $this.clearWxWorkFiles()

        # $ids = $this.getDockerIds();

        $this.showlog("清理完成...")
    
    }

    [void] showlog([string]$mesage) {
        Write-Host $mesage
    }
}

$gnCleaner = [GnCleaner]::new();

$gnCleaner.clear()